--[[
				  Chat Event Monitor
		   written by Torrid - Black Dragonflight server
]]

CEM_VERSION = tonumber(GetAddOnMetadata("ChatEventMonitor", "Version"));

-- defaults
CEM_CFG_DEFAULT = {};
CEM_CFG_DEFAULT.ENABLE = true;
CEM_CFG_DEFAULT.SELECTED = "CHAT_MSG_MONSTER_EMOTE";	-- message type drop down selected
CEM_CFG_DEFAULT.MOVABLE = false;			-- if the windows are moveable or not
CEM_CFG_DEFAULT.LASTMENU = 0;				-- last menu visited.  0 or nil = options, 1 = triggers

CEM_TRIGGER_DEFAULT = {
	string = "",
	name = "",
	timer = 0,
	countdown = false,
	sound = 0,
	disabled = false,
	commands = "",
	lua = false,
	insensitive = false,
	regex = false,
};

CEM_PLAYER = UnitName("player").." "..GetCVar("realmName");
CEM_CFG = {};
CEM_COUNT = nil;				-- the current countdown time remaining.  nil = no countdown
CEM_CURRENTTYPE = "CHAT_MSG_MONSTER_EMOTE";	-- the message type of the trigger currently being viewed/edited
CEM_CURRENTKEY = nil;				-- the key in the array of the currently viewed/edited trigger
CEM_TOTAL = 0;					-- total number of triggers
CEM_LASTSECOND = 0;
CEM_COLOR = {
	"|c00ff0000",
	"|c00ff8800",
	"|c00ffff00",
	"|c0088ff00",
	"|c0000ff00",
}

CEM_MSGTYPE = {
	CHAT_MSG_MONSTER_EMOTE = {
		haystack = "$arg2 $arg1"
	},
	CHAT_MSG_MONSTER_SAY = {
		haystack = string.format(CHAT_MONSTER_SAY_GET, "[$arg2]").."$arg1"
	},
	CHAT_MSG_MONSTER_YELL = {
		haystack = string.format(CHAT_MONSTER_YELL_GET, "[$arg2]").."$arg1"
	},
	CHAT_MSG_RAID = {
		haystack = string.format(CHAT_RAID_GET, "[$arg2]").."$arg1"
	},
	CHAT_MSG_GUILD = {
		haystack = string.format(CHAT_GUILD_GET, "[$arg2]").."$arg1"
	},
	CHAT_MSG_CHANNEL = {
		haystack = "[$arg4] [$arg2]: $arg1"
	},
	CHAT_MSG_SAY = {
		haystack = string.format(CHAT_SAY_GET, "[$arg2]").."$arg1"
	},
	CHAT_MSG_WHISPER = {
		haystack = string.format(CHAT_WHISPER_GET, "[$arg2]").."$arg1"
	},
	CHAT_MSG_PARTY = {
		haystack = string.format(CHAT_PARTY_GET, "[$arg2]").."$arg1"
	},
	CHAT_MSG_OFFICER = {
		haystack = string.format(CHAT_OFFICER_GET, "[$arg2]").."$arg1"
	},
	CHAT_MSG_BATTLEGROUND = {
		haystack = string.format(CHAT_BATTLEGROUND_GET, "[$arg2]").."$arg1"
	},
	CHAT_MSG_SYSTEM = {
		haystack = "$arg1"
	},
	CHAT_MSG_RAID_LEADER = {
		haystack = string.format(CHAT_RAID_LEADER_GET, "[$arg2]").."$arg1"
	},
	CHAT_MSG_BATTLEGROUND_LEADER = {
		haystack = string.format(CHAT_BATTLEGROUND_LEADER_GET, "[$arg2]").."$arg1"
	},
};

CEM_TRIGGERS = {};

CEM_TIMERS = {};

CEM_SOUNDFX = {
	[1] = {
		name = "Horde Bells",
		path = "Sound\\Doodad\\BellTollHorde.wav"
	},
	[2] = {
		name = "Alliance Bells",
		path = "Sound\\Doodad\\BellTollAlliance.wav"
	},
	[3] = {
		name = "Boat Docked Warning",
		path = "Sound\\Doodad\\BoatDockedWarning.wav"
	},
	[4] = {
		name = "Auction Window Open",
		path = "Sound\\interface\\AuctionWindowOpen.wav"
	},
	[5] = {
		name = "Male Gnome Roar",
		path = "Sound\\Character\\PlayerRoars\\CharacterRoarsGnomeMale.wav"
	},
	[6] = {
		name = "Chicken Death",
		path = "Sound\\Creature\\Chicken\\ChickenDeathA.wav"
	},
	[7] = {
		name = "Dryad Death",
		path = "Sound\\Creature\\Dryad\\DryadDeathA.wav"
	},
	[8] = {
		name = "Ogre Aggro 1",
		path = "Sound\\Creature\\Ogre\\mOgreAggro1.wav"
	},
	[9] = {
		name = "Arcane Crystal Open",
		path = "Sound\\Doodad\\ArcaneCrystalOpen.wav"
	},
	[10] = {
		name = "Dwarf Horn",
		path = "Sound\\Doodad\\DwarfHorn.wav"
	},
	[11] = {
		name = "Level Up",
		path = "Sound\\interface\\LevelUp.wav"
	},
	[12] = {
		name = "Level Up 2",
		path = "Sound\\interface\\levelup2.wav"
	},
	[13] = {
		name = "Yarrrr",
		path = "Sound\\Spells\\YarrrrImpact.wav"
	},
	[14] = {
		name = "Interface Button A",
		path = "Sound\\interface\\iUiInterfaceButtonA.wav"
	},
	[15] = {
		name = "Put Down Ring",
		path = "Sound\\interface\\PickUp\\PutDownRing.wav"
	},
	[16] = {
		name = "Spell Icon Drop",
		path = "Sound\\interface\\uSpellIconDrop.wav"
	},
	[17] = {
		name = "Mini Map Zoom",
		path = "Sound\\interface\\uMiniMapZoom.wav"
	},
};

function CEMCopyTable(t)
	local new = {};
	local i, v = next(t, nil);
	while i do
		if (type(v) == "table") then
			v = CEMCopyTable(v);
		end
		new[i] = v;
		i, v = next(t, i);
	end
	return new;
end

function ChatEventMonitor_OnLoad(self)
	SlashCmdList["CEM"] = CEMCmdLine;
	SLASH_CEM1 = "/cem"; 

	-- register trigger events
	local k;
	for k in pairs(CEM_MSGTYPE) do
		self:RegisterEvent(k);
	end

	if (DEFAULT_CHAT_FRAME) then
		DEFAULT_CHAT_FRAME:AddMessage("|c00ffff00Chat Event Monitor|r v|c0000ffff"..CEM_VERSION.."|r AddOn loaded.  |c0000ff00/cem|r to configure");
	end
end

function CEMStartCountdown(t)
	CEM_COUNTER_END = GetTime() + t;
	CEMCountdownFrame:Show();
--	CEMCountdownFrameText:SetText(t);
	CEM_COUNT = t;
end

function CEMCountdown_OnUpdate(self)
	if (CEM_COUNT) then

		local timenow = GetTime();
		if ((CEM_COUNTER_END - CEM_COUNT) + .1 < timenow) then
			CEM_COUNT = CEM_COUNTER_END - timenow;

			CEMCountdownFrameText:SetText(string.format("%i", CEM_COUNT));

			if (CEM_COUNT < 1) then
				CEM_COUNT = nil;
				CEMCountdownFrameText:SetText("");

				self:Hide();
			end
		end
	end
end

function CEMFormatTime(endstamp)
	local h, m, s;
	local time = endstamp - GetTime();
	local text;
	
	if (time <= 0) then
		return "00:00";
	elseif (time < 3600) then
		_, h, m, s = ChatFrame_TimeBreakDown(time);
		return format("%02d:%02d", m, s);
	elseif (time < 86400) then
		_, h, m, s = ChatFrame_TimeBreakDown(time);
		return format("%02d:%02d:%02d", h, m, s);
	else
		return "> 24 hrs";
	end
end

function CEMTimersDisplayFrame_OnUpdate()
	if (CEM_TIMERS[1]) then
		local i, timer, text, remaining;
		local timeNow = GetTime();

		text = "";

		for i, timer in ipairs(CEM_TIMERS) do
			if (timer.endstamp < timeNow + 1) then
				-- timer expired.  delete
				table.remove(CEM_TIMERS, i);
			else
				remaining = timer.endstamp - timeNow;

				if (remaining < 11) then
					text = text..CEM_COLOR[1];
					if (not CEM_COUNT and timer.countdown) then
						CEMStartCountdown(remaining);
					end
				elseif (remaining <= 60) then
					text = text..CEM_COLOR[2];
				else
					text = text..CEM_COLOR[3];
				end

				text = text.."["..CEMFormatTime(timer.endstamp).."] "..timer.name.."|r\n";

			end
		end

		CEMTimersDisplayFrameText:SetText(text);
	end
end

function CEMAddTimer(trigger)
	table.insert(CEM_TIMERS, {
		name = trigger.name,
		endstamp = GetTime() + trigger.timer + 1,
		countdown = trigger.countdown
	});

	table.sort(CEM_TIMERS, 
		function(a, b)
			return a.endstamp < b.endstamp;
		end
	);

end

function CEMOptionsFrame_OnEvent(event)
	if (event == "VARIABLES_LOADED") then

		local k, v, i, k2;

		CEM_PLAYER = UnitName("player").." "..GetCVar("realmName");

		-- new character?
		if (not CEM_CFG[CEM_PLAYER]) then
			CEM_CFG[CEM_PLAYER] = {};
			CEM_TRIGGERS[CEM_PLAYER] = {};
		end

		-- make sure CFG array has same keys as default
		for k, v in pairs(CEM_CFG_DEFAULT) do
			if (type(CEM_CFG[CEM_PLAYER][k]) == "nil") then
				CEM_CFG[CEM_PLAYER][k] = v;
			end
		end

		-- make sure every trigger has same keys as default (new versions may add keys)
		for k in pairs(CEM_TRIGGERS[CEM_PLAYER]) do
			for i in pairs(CEM_TRIGGERS[CEM_PLAYER][k]) do
				for k2, v in pairs(CEM_TRIGGER_DEFAULT) do
					if (type(CEM_TRIGGERS[CEM_PLAYER][k][i][k2]) == "nil") then
						CEM_TRIGGERS[CEM_PLAYER][k][i][k2] = v;
					end
				end
			end
		end

		UIDropDownMenu_SetSelectedValue(CEMMessageTypeDropDown, CEM_CFG[CEM_PLAYER].SELECTED);

		-- get total number of triggers
		CEM_TOTAL = 0;
		for _, k in pairs(CEM_TRIGGERS[CEM_PLAYER]) do
			for _ in pairs(k) do
				CEM_TOTAL = CEM_TOTAL + 1;
			end
		end

	end
end

function CEMTriggersEventHandler(event, ...)

	if ( not CEM_CFG[CEM_PLAYER].ENABLE ) then
		return
	end

	-- === Trigger Handling ======================================================

	local arg1, arg2, arg3, arg4 = select(1, ...);
	local haystack;
	local k, v;
	local match, ends;

	-- some messages are split into several args
	-- this is to make the searched string exactly like the player sees it

	-- this way seems to be the least processor intensive
	if (CEM_MSGTYPE[event].haystack == "$arg1") then
		-- _SPELL_ and _COMBAT_ events
		haystack = arg1;
	elseif (CEM_MSGTYPE[event].haystack == "$arg2 $arg1") then
		-- EMOTE events
		haystack = arg2.." "..arg1;
	else
		-- CHAT_MSG_RAID, _SAY, etc events with multiple args
		arg1 = string.gsub(arg1, "%%", "%%%%");
		haystack = string.gsub(CEM_MSGTYPE[event].haystack, "%$arg1", arg1);
		if (arg2) then
			haystack = string.gsub(haystack, "%$arg2", arg2);
		end
		if (arg3) then
			haystack = string.gsub(haystack, "%$arg3", arg3);
		end
		if (arg4) then
			-- ChatFrame.lua does this, so I must.  This strips away the " - Zone"
			-- from the channel name, if there is one (i.e. General and Trade)
			arg4 = gsub(arg4, "%s%-%s.*", "");
			haystack = string.gsub(haystack, "%$arg4", arg4);
		end

		-- just in case
		if (not haystack) then
			haystack = arg1;
		end
	end

	-- if triggers for this message type, check for matches
	if (CEM_TRIGGERS[CEM_PLAYER][event]) then
		for k, v in pairs(CEM_TRIGGERS[CEM_PLAYER][event]) do

			if (v.insensitive) then
				if (v.regex) then
					match, ends = string.find(string.lower(haystack), string.lower(v.string), 1, false);
				else
					match, ends = string.find(string.lower(haystack), string.lower(v.string), 1, true);
				end
			else
				if (v.regex) then
					match, ends = string.find(haystack, v.string, 1, false);
				else
					match, ends = string.find(haystack, v.string, 1, true);
				end
			end

			-- if match found, and trigger text is not empty, and trigger is enabled
			if (match and v.string ~= "" and not v.disabled) then

				-- sound effect for triggers
				if (v.sound > 0) then
					PlaySoundFile(CEM_SOUNDFX[v.sound].path);
				end

				-- timer for triggers
				if (v.timer > 0) then
					-- CEMStartCountdown(v.timer);
					CEMAddTimer(v);
				end

				-- capture target
				-- do this here so target can be used in commands
				local target;
				if (v.capture == 0) then
					target = "";
				elseif (v.capture == 1) then
					_, _, target = string.find(string.sub(haystack, 1, match - 1), "%s*(%w+)%s*$");
				elseif (v.capture == 2) then
					_, _, target = string.find(string.sub(haystack, ends + 1, -1), "^%s*(%w+)%s*");
				else
					target = "_SELF";
				end
				if (target == nil) then
					target = "";
				end
				if (target ~= "_SELF") then
					CEM_TARGET = target;
				else
					CEM_TARGET = UnitName("player");
				end

				-- commands
				if (v.commands ~= "") then
					if (v.lua) then
						RunScript(v.commands);
					else
						local s;
						for s in string.gmatch(v.commands, "[^\n]+") do
							-- $target
							s = string.gsub(s, "%$target", CEM_TARGET);
							CEMCommandEditBox:SetText(s);
							ChatEdit_SendText(CEMCommandEditBox);
						end
					end
				end

			end
		end

	end
end

function CEMCmdLine(arg)
	_, _, arg1, arg2 = string.find(arg, "^%s*(%S+)%s+(.+)$");
	if (not arg1) then
		arg1 = arg;
	end
	arg1 = string.lower(arg1);

	if (arg1 == "enable" and not arg2) then
		-- global trigger enable
		CEM_CFG[CEM_PLAYER].ENABLE = true;
		DEFAULT_CHAT_FRAME:AddMessage("All triggers enabled.", 1, 1, 0);
	elseif (arg1 == "enable" and arg2) then
		-- individual trigger enable
		local i, type, trigger, found;

		found = 0;
		for type in pairs(CEM_TRIGGERS[CEM_PLAYER]) do
			for i, trigger in pairs(CEM_TRIGGERS[CEM_PLAYER][type]) do
				if (trigger.name == arg2) then
					CEM_TRIGGERS[CEM_PLAYER][type][i].disabled = false;
					found = found + 1;
				end
			end
		end
		if (found == 0) then
			DEFAULT_CHAT_FRAME:AddMessage("No triggers with the name '"..arg2.."' found", 1, 0, 0);
		else
			DEFAULT_CHAT_FRAME:AddMessage("All triggers named '"..arg2.."' enabled ("..found..")", 1, 1, 0);
		end
	elseif (arg1 == "disable" and not arg2) then
		-- global trigger disable
		CEM_CFG[CEM_PLAYER].ENABLE = false;
		DEFAULT_CHAT_FRAME:AddMessage("All triggers disabled.", 1, 1, 0);
	elseif (arg1 == "disable" and arg2) then
		-- individual trigger disable
		local i, type, trigger, found;

		found = 0;
		for type in pairs(CEM_TRIGGERS[CEM_PLAYER]) do
			for i, trigger in pairs(CEM_TRIGGERS[CEM_PLAYER][type]) do
				if (trigger.name == arg2) then
					CEM_TRIGGERS[CEM_PLAYER][type][i].disabled = true;
					found = found + 1;
				end
			end
		end
		if (found == 0) then
			DEFAULT_CHAT_FRAME:AddMessage("No triggers with the name '"..arg2.."' found", 1, 0, 0);
		else
			DEFAULT_CHAT_FRAME:AddMessage("All triggers named '"..arg2.."' disabled ("..found..")", 1, 0, 0);
		end
	elseif (arg1 == "removetimer" and arg2) then
		-- remove timer by name
		local i, timer, found;

		found = 0;
		for i, timer in pairs(CEM_TIMERS) do
			if (timer.name == arg2) then
				table.remove(CEM_TIMERS, i);
				found = found + 1;
				CEMTimersDisplayFrameText:SetText("");
				break;
			end
		end
		if (found == 0) then
			DEFAULT_CHAT_FRAME:AddMessage("There are no timers with the name '"..arg2.."'", 1, 0, 0);
		else
			DEFAULT_CHAT_FRAME:AddMessage("Timer '"..arg2.."' removed", 1, 0, 0);
		end

	elseif (arg1 == "?" or arg1 == "help") then
		DEFAULT_CHAT_FRAME:AddMessage("Chat Event Monitor v"..CEM_VERSION.." Commands:", 1, 1, 0);
		DEFAULT_CHAT_FRAME:AddMessage("/cem -- Options/Trigger editor");
		DEFAULT_CHAT_FRAME:AddMessage("/cem enable -- Enables all triggers");
		DEFAULT_CHAT_FRAME:AddMessage("/cem enable <name> -- Enable trigger by name");
		DEFAULT_CHAT_FRAME:AddMessage("/cem disable -- Disables all triggers");
		DEFAULT_CHAT_FRAME:AddMessage("/cem disable <name> -- Disable trigger by name");
		DEFAULT_CHAT_FRAME:AddMessage("/cem removetimer <name> -- Remove timer by name");
	else
		if (CEM_CFG[CEM_PLAYER].LASTMENU == 1) then
			CEMEditTriggersFrame:Show();
		else
			CEMOptionsFrame:Show();
		end
		PlaySound("igMainMenuOpen");
	end
end

------------------- Message Type Drop Down ----------------------

function CEMMessageTypeDropDown_OnLoad(self)
	UIDropDownMenu_Initialize(self, CEMMessageTypeDropDown_Initialize);
	UIDropDownMenu_SetSelectedValue(CEMMessageTypeDropDown, CEM_CFG_DEFAULT.SELECTED);
	UIDropDownMenu_SetWidth(CEMMessageTypeDropDown, 160);
end

function CEMMessageTypeDropDown_Initialize()
	local k, v, info;
	local i, text;

	-- CHAT_MSG_CHANNEL doesn't have a GlobalStrings.lua entry
	if (not CHAT_MSG_CHANNEL) then
		CHAT_MSG_CHANNEL = "Chat Channels"
	end

	for k, v in pairs(CEM_MSGTYPE) do
		i = 0;
		if (CEM_TRIGGERS[CEM_PLAYER] and CEM_TRIGGERS[CEM_PLAYER][k]) then
			for _ in pairs(CEM_TRIGGERS[CEM_PLAYER][k]) do
				i = i + 1;
			end
			text = " ("..i..")";
		else
			text = "";
		end

		info = {};
		info.text = getglobal(k)..text;		-- GlobalStrings.lua and # of triggers
		info.value = k;
		info.func = CEMMessageTypeDropDown_OnClick;
		UIDropDownMenu_AddButton(info);
	end
end

function CEMMessageTypeDropDown_OnClick(self)
	UIDropDownMenu_SetSelectedValue(CEMMessageTypeDropDown, self.value);
	CEM_CFG[CEM_PLAYER].SELECTED = UIDropDownMenu_GetSelectedValue(CEMMessageTypeDropDown);
	
	-- save currently viewed trigger, if one
	CEMSaveEditBoxData();

	-- set currently viewing message type to newly selected
	CEM_CURRENTTYPE = CEM_CFG[CEM_PLAYER].SELECTED;

	-- if this type has at least one trigger, bring up first one
	if (not CEM_TRIGGERS[CEM_PLAYER][CEM_CURRENTTYPE] or not next(CEM_TRIGGERS[CEM_PLAYER][CEM_CURRENTTYPE])) then
		CEM_CURRENTKEY = nil;
	else
		CEM_CURRENTKEY = 1;
	end

	CEMEditTriggersUpdate();
end

-------------------- Sound Effect Drop Down ---------------------

function CEMSoundFXDropDown_OnLoad(self)
	UIDropDownMenu_Initialize(self, CEMSoundFXDropDown_Initialize);
	UIDropDownMenu_SetSelectedValue(CEMSoundFXDropDown, 0);
	UIDropDownMenu_SetWidth(CEMSoundFXDropDown, 125);
end

function CEMSoundFXDropDown_Initialize()
	local k, v, info;

	-- no sound effect option
	info = {};
	info.text = "No Sound Effect";
	info.value = 0;
	info.func = CEMSoundFXDropDown_OnClick;
	UIDropDownMenu_AddButton(info);

	for k, v in pairs(CEM_SOUNDFX) do
		info = {};
		info.text = v.name;
		info.value = k;
		info.func = CEMSoundFXDropDown_OnClick;
		UIDropDownMenu_AddButton(info);
	end
end

function CEMSoundFXDropDown_OnClick(self)
	UIDropDownMenu_SetSelectedValue(CEMSoundFXDropDown, self.value);
	if (CEM_CURRENTKEY) then
		CEM_TRIGGERS[CEM_PLAYER][CEM_CURRENTTYPE][CEM_CURRENTKEY].sound = UIDropDownMenu_GetSelectedValue(CEMSoundFXDropDown);
	end

	-- play button
	if (CEM_TRIGGERS[CEM_PLAYER][CEM_CURRENTTYPE][CEM_CURRENTKEY].sound > 0) then
		CEMPlayButton:Enable();
	else
		CEMPlayButton:Disable();
	end
end

----------------- OnShow and Update ----------------------------

function CEMEditTriggersFrame_OnShow()
	-- set current msg type to the index in cfg.selected
	CEM_CURRENTTYPE = CEM_CFG[CEM_PLAYER].SELECTED;
	UIDropDownMenu_SetSelectedValue(CEMMessageTypeDropDown, CEM_CFG[CEM_PLAYER].SELECTED);

	-- if no triggers for this type
	if (not CEM_TRIGGERS[CEM_PLAYER][CEM_CURRENTTYPE]) then
		CEM_CURRENTKEY = nil;
	elseif (not CEM_CURRENTKEY or not CEM_TRIGGERS[CEM_PLAYER][CEM_CURRENTTYPE][CEM_CURRENTKEY]) then
		-- get first trigger
		CEM_CURRENTKEY = 1;
	end

	CEMEditTriggersUpdate();
end

function CEMSetCommandPreviewText(text)
	local s = {};
	local i = 1;
	local preview = "";

	CEMEditTriggersFrameCommandsText1:SetText("");
	CEMEditTriggersFrameCommandsText2:SetText("");
	CEMEditTriggersFrameCommandsText3:SetText("");
	CEMEditTriggersFrameCommandsText4:SetText("");

	for s in string.gmatch(text, "[^\n]+") do
		if (i > 4) then
			CEMEditTriggersFrameCommandsText4:SetText("...");
			break;
		end
		getglobal("CEMEditTriggersFrameCommandsText"..i):SetText(s);
		i = i + 1;
	end
	
	return preview;
end

function CEMEditTriggersUpdate()
	CEMCommandEditFrame:Hide();

	-- figure out what buttons need to be grayed and get currently selected trigger variables for editing
	if (not CEM_CURRENTKEY) then
		-- there are no triggers for this msg type.  gray most everything

		UIDropDownMenu_SetSelectedValue(CEMSoundFXDropDown, 0);
		CEMTriggerTextEditBox:SetText("");
		CEMTimerHEditBox:SetText("");
		CEMTimerMEditBox:SetText("");
		CEMTimerSEditBox:SetText("");
		CEMTriggerNameEditBox:SetText("");

		CEMNextPageButton:Disable();
		CEMPrevPageButton:Disable();
		CEMDeleteButton:Disable();
		CEMPlayButton:Disable();

		UIDropDownMenu_DisableDropDown(CEMSoundFXDropDown);
		CEMTriggerTextEditBox:Hide();
		CEMTimerHEditBox:Hide();
		CEMTimerMEditBox:Hide();
		CEMTimerSEditBox:Hide();
		CEMTriggerNameEditBox:Hide();

		CEMSetCommandPreviewText("");
		CEMCommandEditFrameEditBox:SetText("");
		CEMCommandsEditButton:Disable();

--		OptionsFrame_DisableCheckBox(CEMTriggerEditRegExCheckButton);
--		OptionsFrame_DisableCheckBox(CEMTriggerEditCaseCheckButton);
--		OptionsFrame_DisableCheckBox(CEMTriggerEditDisabledCheckButton);
--		OptionsFrame_DisableCheckBox(CEMTriggerEditCountdownCheckButton);
		CEMTriggerEditRegExCheckButton:Disable();
		CEMTriggerEditCaseCheckButton:Disable();
		CEMTriggerEditDisabledCheckButton:Disable();
		CEMTriggerEditCountdownCheckButton:Disable();

		CEMTriggerEditRegExCheckButton:SetChecked(nil);
		CEMTriggerEditDisabledCheckButton:SetChecked(nil);
		CEMTriggerEditCountdownCheckButton:SetChecked(nil);
		CEMTriggerEditCaseCheckButton:SetChecked(nil);

		CEMEditFrameIdLabel:SetText("Triggers: "..CEM_TOTAL.."\n0 of this type");
	else
		-- # of triggers for this type
		local triggers = table.getn(CEM_TRIGGERS[CEM_PLAYER][CEM_CURRENTTYPE]);
		
		-- Prev button
		if (CEM_CURRENTKEY == 1) then
			CEMPrevPageButton:Disable();
		else
			CEMPrevPageButton:Enable();
		end

		-- Next button
		if (triggers > CEM_CURRENTKEY) then
			CEMNextPageButton:Enable();
		else
			CEMNextPageButton:Disable();
		end

		-- Delete button
		CEMDeleteButton:Enable();

		-- Sound Effect Drop Down
		UIDropDownMenu_EnableDropDown(CEMSoundFXDropDown);
		UIDropDownMenu_SetSelectedValue(CEMSoundFXDropDown, CEM_TRIGGERS[CEM_PLAYER][CEM_CURRENTTYPE][CEM_CURRENTKEY].sound);

		-- play button and sound effect drop down set text
		if (CEM_TRIGGERS[CEM_PLAYER][CEM_CURRENTTYPE][CEM_CURRENTKEY].sound > 0) then
			CEMPlayButton:Enable();

			-- The drop down button ends up displaying weird text without this for some reason
			UIDropDownMenu_SetText(CEMSoundFXDropDown, CEM_SOUNDFX[CEM_TRIGGERS[CEM_PLAYER][CEM_CURRENTTYPE][CEM_CURRENTKEY].sound].name);
		else
			CEMPlayButton:Disable();
			UIDropDownMenu_SetText(CEMSoundFXDropDown, "No Sound Effect");
		end

		-- Text Edit Box
		CEMTriggerTextEditBox:Show();
		CEMTriggerTextEditBox:SetText(CEM_TRIGGERS[CEM_PLAYER][CEM_CURRENTTYPE][CEM_CURRENTKEY].string);

		-- Name Edit Box
		CEMTriggerNameEditBox:Show();
		CEMTriggerNameEditBox:SetText(CEM_TRIGGERS[CEM_PLAYER][CEM_CURRENTTYPE][CEM_CURRENTKEY].name);

		-- Timer Edit Box
		local _, h, m, s = ChatFrame_TimeBreakDown(CEM_TRIGGERS[CEM_PLAYER][CEM_CURRENTTYPE][CEM_CURRENTKEY].timer);
		CEMTimerHEditBox:Show();
		CEMTimerMEditBox:Show();
		CEMTimerSEditBox:Show();
		CEMTimerHEditBox:SetText(h);
		CEMTimerMEditBox:SetText(m);
		CEMTimerSEditBox:SetText(s);

		-- Commands
		CEMSetCommandPreviewText(CEM_TRIGGERS[CEM_PLAYER][CEM_CURRENTTYPE][CEM_CURRENTKEY].commands);
		CEMCommandEditFrameEditBox:SetText(CEM_TRIGGERS[CEM_PLAYER][CEM_CURRENTTYPE][CEM_CURRENTKEY].commands);
		CEMCommandsEditButton:Enable();
		if (CEM_TRIGGERS[CEM_PLAYER][CEM_CURRENTTYPE][CEM_CURRENTKEY].lua) then
			CEMExecuteLuaCheckButton:SetChecked(1);
		else
			CEMExecuteLuaCheckButton:SetChecked(nil);
		end

		-- check boxes
		--OptionsFrame_EnableCheckBox(CEMTriggerEditRegExCheckButton);
		CEMTriggerEditRegExCheckButton:Enable();
		if (CEM_TRIGGERS[CEM_PLAYER][CEM_CURRENTTYPE][CEM_CURRENTKEY].regex) then
			CEMTriggerEditRegExCheckButton:SetChecked(1);
		else
			CEMTriggerEditRegExCheckButton:SetChecked(nil);
		end

		--OptionsFrame_EnableCheckBox(CEMTriggerEditDisabledCheckButton);
		CEMTriggerEditDisabledCheckButton:Enable();
		if (CEM_TRIGGERS[CEM_PLAYER][CEM_CURRENTTYPE][CEM_CURRENTKEY].disabled) then
			CEMTriggerEditDisabledCheckButton:SetChecked(1);
		else
			CEMTriggerEditDisabledCheckButton:SetChecked(nil);
		end

		--OptionsFrame_EnableCheckBox(CEMTriggerEditCountdownCheckButton);
		CEMTriggerEditCountdownCheckButton:Enable();
		if (CEM_TRIGGERS[CEM_PLAYER][CEM_CURRENTTYPE][CEM_CURRENTKEY].countdown) then
			CEMTriggerEditCountdownCheckButton:SetChecked(1);
		else
			CEMTriggerEditCountdownCheckButton:SetChecked(nil);
		end

		--OptionsFrame_EnableCheckBox(CEMTriggerEditCaseCheckButton);
		CEMTriggerEditCaseCheckButton:Enable();
		if (CEM_TRIGGERS[CEM_PLAYER][CEM_CURRENTTYPE][CEM_CURRENTKEY].insensitive) then
			CEMTriggerEditCaseCheckButton:SetChecked(1);
		else
			CEMTriggerEditCaseCheckButton:SetChecked(nil);
		end

		CEMEditFrameIdLabel:SetText("Triggers: "..CEM_TOTAL.."\nViewing "..CEM_CURRENTKEY.." of "..triggers);
	end

end

------------------- Buttons -------------------------------------

function CEMNewButton_OnClick()
	local new = CEMCopyTable(CEM_TRIGGER_DEFAULT);

	-- if triggers array doesn't have a key of this type, make it
	if (not CEM_TRIGGERS[CEM_PLAYER][CEM_CURRENTTYPE]) then
		CEM_TRIGGERS[CEM_PLAYER][CEM_CURRENTTYPE] = {};
	end

	-- create new trigger and jump to it
	table.insert(CEM_TRIGGERS[CEM_PLAYER][CEM_CURRENTTYPE], new);
	CEM_CURRENTKEY = table.getn(CEM_TRIGGERS[CEM_PLAYER][CEM_CURRENTTYPE]);

	CEM_TOTAL = CEM_TOTAL + 1;

	-- update drop down
	UIDropDownMenu_Initialize(CEMMessageTypeDropDown, CEMMessageTypeDropDown_Initialize);
	UIDropDownMenu_SetSelectedValue(CEMMessageTypeDropDown, CEM_CFG[CEM_PLAYER].SELECTED);

	CEMEditTriggersUpdate();

	CEMTriggerTextEditBox:SetFocus();
end

function CEMDeleteButton_OnClick()
	table.remove(CEM_TRIGGERS[CEM_PLAYER][CEM_CURRENTTYPE], CEM_CURRENTKEY);
	if (table.getn(CEM_TRIGGERS[CEM_PLAYER][CEM_CURRENTTYPE]) == 0) then
		CEM_CURRENTKEY = nil;
		-- delete key from triggers array since it has no more triggers to keep it clean
		CEM_TRIGGERS[CEM_PLAYER][CEM_CURRENTTYPE] = nil;
	elseif (CEM_CURRENTKEY > 1) then
		CEM_CURRENTKEY = CEM_CURRENTKEY - 1;
	end

	CEM_TOTAL = CEM_TOTAL - 1;

	-- update drop down
	UIDropDownMenu_Initialize(CEMMessageTypeDropDown, CEMMessageTypeDropDown_Initialize);
	UIDropDownMenu_SetSelectedValue(CEMMessageTypeDropDown, CEM_CFG[CEM_PLAYER].SELECTED);

	CEMEditTriggersUpdate();
end

-------------------- Save --------------------------------------

function CEMSaveEditBoxData()
	if (CEM_CURRENTKEY) then
		local h, m, s;

		h = tonumber(CEMTimerHEditBox:GetText());
		m = tonumber(CEMTimerMEditBox:GetText());
		s = tonumber(CEMTimerSEditBox:GetText());
		if (not h) then
			h = 0;
		end
		if (not m) then
			m = 0;
		end
		if (not s) then
			s = 0;
		end

		if (h > 23) then
			h = 23;
			CEMTimerHEditBox:SetText("23");
		end
		if (m > 59) then
			m = 59;
			CEMTimerMEditBox:SetText("59");
		end
		if (s > 59) then
			s = 59;
			CEMTimerSEditBox:SetText("59");
		end

		CEM_TRIGGERS[CEM_PLAYER][CEM_CURRENTTYPE][CEM_CURRENTKEY].timer = (h * 60 * 60) + (m * 60) + s;
		CEM_TRIGGERS[CEM_PLAYER][CEM_CURRENTTYPE][CEM_CURRENTKEY].string = CEMTriggerTextEditBox:GetText();
		CEM_TRIGGERS[CEM_PLAYER][CEM_CURRENTTYPE][CEM_CURRENTKEY].name = CEMTriggerNameEditBox:GetText();
	end
end
